import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Defines cats, which are animals that dance, eat, sleep, walk, and jump. 
 * 
 * @author Michael Kolling
 * @author Mihaela Sabin
 *      - Cat extends Animal to inherit 'canSee' and 'remove' methods and  
 *          to pass them to subclass FriendlyCat in friendly-cat scenario.
 *      - added 'jumpUp' and 'jumpDown', which use 'jump', coded after 'walk'
 *      - resized cat images to half of their original sizes in 'fatcat' scenario
 * @authour add your name here
 * @version 1 - Sep 10, 2010
 */
public class Cat extends Animal
{
    private boolean sleepy = false;
    private boolean hungry = false;
    private boolean bored = true;
    
    private int pizzasEaten; 
    private int poisonapplesEaten;
    private GreenfootImage catImg;
    public void act()
    {
        if(foundPizza()) {
            eatPizza();
        }
        if(foundPoisonApple()) {
            eatPoisonApple();
        }
    }
    public int getPizzasEaten()
    {
         return pizzasEaten;
    }
    public int getPoisonApplesEaten()
    {
        return poisonapplesEaten;
    }
    /**
     * Create a cat and initialize its images
     */
    public Cat( ) 
    {
        catImg = new GreenfootImage("small-cat.png");
        this.setImage(catImg);
        pizzasEaten = 0;
        poisonapplesEaten = 0;
    }    
    public void eatPizza()
    {
        Actor pizza = getOneObjectAtOffset(0, 0, Pizza.class);        
        if(pizza != null) {
            // eat the pizza...
            getWorld().removeObject(pizza);
            pizzasEaten = pizzasEaten + 1;            
        }
        
    }
    public boolean foundPizza()
    {
        Actor pizza = getOneObjectAtOffset(0, 0, Pizza.class);
        if(pizza != null) {
            return true;
        }
        else {
            return false;
        }
    }
    public void eatPoisonApple()
    {
        Actor poisonapple = getOneObjectAtOffset(0, 0, PoisonApple.class);  
        int x = Greenfoot.getRandomNumber(559);
        int y = Greenfoot.getRandomNumber(559);
        if(poisonapple != null) {
            // eat the poisonapple...
            getWorld().removeObject(poisonapple);
            poisonapplesEaten = poisonapplesEaten + 1;  
           getWorld().addObject(new PoisonApple(), x, y);
            if (poisonapplesEaten == 1)
            {
                sleep(10);
                poisonapplesEaten = 0;
            }
        
        }
    }
    public boolean foundPoisonApple()
    {
        Actor poisonapple = getOneObjectAtOffset(0, 0, PoisonApple.class);
        if(poisonapple != null) {
            return true;
        }
        else {
            return false;
        }
    }
    /**
     * Walk a bit to the left. 'distance' determines how far to walk. Use small numbers (1 to 10).
     */
    public void walkLeft(int distance)
    {
        this.walk(distance, -10, "small-cat-walk.png", "small-cat-walk-2.png");
    }
    
    /**
     * Walk a bit to the right. 'distance' determines how far to walk. Use small numbers (1 to 10).
     */
    public void walkRight(int distance)
    {
        this.walk(distance, 10, "small-cat-walk-right.png", "small-cat-walk-right-2.png");
    }
    
    /**
     * Jump up. 'distance' determines how high to jump. Use small numbers (1 to 10).
     */
    public void jumpUp(int distance)
    {
        this.jump(distance, -10, "small-cat-fly.png", "small-cat-fly-2.png");
    }
    
    /**
     * Jump down. 'distance' determines how far down to jump. Use small numbers (1 to 10).
     */
    public void jumpDown(int distance)
    {
      this.jump(distance, 10, "small-cat-fall.png", "small-cat-fall-2.png");
    }
    
    /**
     * Internal walk method. Walk a given distance into a given direction, using given images.
     */
    private void walk(int distance, int direction, String img1, String img2)
    {
        for (int i = 0; i < distance; i++) 
        {
            super.setImage(img1);
            this.wait(4);
            super.setLocation(super.getX( ) + direction, super.getY());
            super.setImage(img2);
            this.wait(4);
            super.setLocation(super.getX() + direction, super.getY());
        }
    }

    /**
     * Internal jump method. Jump a given distance into a given direction, using given images.
     */
    private void jump(int distance, int direction, String img1, String img2)
    {
        super.setImage(img1);
            this.wait(4);
            super.setLocation(super.getX( ), super.getY() + direction);
            super.setImage(img2);
            this.wait(4);
            super.setLocation(super.getX( ), super.getY() + direction);
    }

    /**
     * Do a dance. Cool, Baby!
     * (Dancing makes you sleepy and hungry.)
     */
    public void dance()
    {
        Greenfoot.playSound("music.wav");
        for (int i = 0; i < 6; i++) 
        {
            super.setImage("small-cat-dance.png");
            this.wait(10);
            super.setImage("small-cat.png");
            this.wait(8);
            super.setImage("small-cat-dance-2.png");
            this.wait(8);
            super.setImage("small-cat.png");
            this.wait(8);
        }
        for (int i = 0; i < 5; i++) 
        {
            super.setImage("small-cat-dance.png");
            this.wait(8);
            super.setImage("small-cat-dance-2.png");
            this.wait(6);
        }
        super.setImage("small-cat.png");
        this.sleepy = true;
        this.hungry = true;
        this.bored = false;
    }
        
    /**
     * It's really what the method name says: shout "Hooray", which makes you not bored.
     */
    public void shoutHooray()
    {
        super.setImage("cat-speak.png");
        Greenfoot.playSound("hooray.wav");
        this.wait(20);
        super.setImage("small-cat.png");
        this.bored = false;
    }
    
    /**
     * Sleep for a while. The parameter determines how long to sleep. Use small numbers. 
     * A value of 1 will sleep for a couple of seconds or so.
     * Sleeping makes you not sleepy, but bored. 
     */
    public void sleep(int howLong)
    {
        for (int i = 0; i < howLong; i++) 
        {
            for (int j = 1; j <= 4; j++) 
            {
                super.setImage("small-cat-sleep-" + j + ".png");
                this.wait(10);
            }
        }
        super.setImage("small-cat.png");
        this.sleepy = false;
        this.bored = true;
    }
    
    /**
     * Eat some pizza!
     * Eating makes you sleepy (but not hungry anymore).
     */
    public void eat()
    {
        for (int i = 0; i < 4; i++) 
        {
            super.setImage("small-cat-eat.png");
            this.wait(8);
            super.setImage("small-cat-eat-2.png");
            this.wait(6);
        }
        super.setImage("small-cat.png");
        this.sleepy = true;
        this.hungry = false;
    }
       
    /**
     * Return true if the cat is alone here.
     */
    public boolean isAlone()
    {
        int numberOfCats = super.getWorld().getObjects(Cat.class).size();
        return numberOfCats < 2;
    }
    
    /**
     * Return true if the cat is not alone here.
     */
    public boolean hasCompany()
    {
        return !this.isAlone();
    }
    
    /**
     * Return true if the cat is hungy.
     */
    public boolean isHungry()
    {
        return this.hungry;
    }
    
    /**
     * Return true if the cat is sleepy.
     */
    public boolean isSleepy()
    {
        return this.sleepy;
    }
    
    /**
     * Return true if the cat is bored.
     */
    public boolean isBored()
    {
        return this.bored;
    }
    
    /**
     * Wait for a given time.
     */
    public void wait(int time)
    {
        Greenfoot.delay(time);
    }

}
